//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Threading;

namespace ProfilingSamples
{
	public class FinallyMain 
	{
		private void FinallyFunction()
		{
			try
			{
				TimeSpan ts = new TimeSpan(Environment.TickCount);
				throw new ApplicationException(string.Format("{0} . . .", ts.ToString()));
			}
			finally
			{
				Console.WriteLine("finally");
			}
		}
		private void FinallyThread()
		{
			while(true)
			{
				try
				{
					FinallyFunction();
				}
				catch(ApplicationException ae)
				{
					Console.WriteLine("FinallyThread restarting: {0}", ae.Message);
				}
				catch(Exception e)
				{
					Console.WriteLine("FinallyThread aborting: {0}", e.Message);
					return;
				}
				// Every half a second I throw an exception.
				Thread.Sleep(500);
			}
		}
		private void TestThread()
		{
			Thread re = new Thread(new ThreadStart(FinallyThread));
			try
			{
				re.Start();
				Thread.Sleep(Timeout.Infinite);
			}
			catch(Exception e)
			{
				Console.WriteLine("TestThread aborting: {0}", e.Message);
				re.Abort();
			}
		}
		static void Main(string [] args)
		{
			FinallyMain m = new FinallyMain();
			Thread testThread = new Thread(new ThreadStart(m.TestThread));
			testThread.Start();
			string input = Console.ReadLine();
			testThread.Abort();
			testThread.Join();
		}
	}
}