//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Threading;

namespace ProfilingSamples
{
	public class LogicalThreadMain 
	{
		private ManualResetEvent [] asyncEvents;
		const int threadCount = 100;
		public LogicalThreadMain()
		{
			asyncEvents = new ManualResetEvent[threadCount];
			for(int i = 0;i < threadCount;i++)
			{
				asyncEvents[i] = new ManualResetEvent(false);
			}
		}
		void WorkerThread(Object number) 
		{
			int id = (int)number;
			// Sleep for 1 second to simulate doing work
			Thread.Sleep(1000);
			// Signal that the async operation is now complete.
			TimeSpan ts = new TimeSpan(Environment.TickCount);
			Console.WriteLine("{0} {1}", id, ts);
			asyncEvents[id].Set();
		}
		void TestThread()
		{
			try
			{
				while(true)
				{ 
					Console.WriteLine("Start ----------------");
					for(int i = 0;i < threadCount;i++)
					{
						ThreadPool.QueueUserWorkItem(new WaitCallback(WorkerThread), i);
					}
					Console.WriteLine("End ----------------");
					for(int i = 0;i < threadCount;i++)
					{
						asyncEvents[i].WaitOne();
						asyncEvents[i].Reset();
					}
				}
			}
			catch(Exception e)
			{
				Console.WriteLine("TestThread aborting: {0}", e.Message);
			}
		}
		static void Main(string [] args)
		{
			LogicalThreadMain m = new LogicalThreadMain();
			Thread testThread = new Thread(new ThreadStart(m.TestThread));
			testThread.Start();
			string input = Console.ReadLine();
			testThread.Abort();
			testThread.Join();
		}
	}
}