//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Xml;
using System.Diagnostics;

namespace WinGcp
{
	/// <summary>
	/// Summary description for ProfileInfo.
	/// </summary>
	public class ProfileInfo
	{
		public delegate void ThreadInfo(uint id, uint handle, uint w32id);
		public delegate void AppDomainInfo(uint id, string name, uint processid);
        public delegate void AssemblyInfo(uint id, string name, uint moduleid, uint appdomainid);
		public delegate void ModuleInfo(uint id, string name, uint assemblyid, long baseaddress);
		public delegate void ClassInfo(uint id, string name, uint token, uint moduleid);
		public delegate void FunctionInfo(uint id, string name, uint token, uint classid,
		                                  uint moduleid, uint size, long startaddress,
							              uint enter, uint leave, uint tail, uint unwind);
		public ThreadInfo threadDelegate;
		public AppDomainInfo appdomainDelegate;
		public AssemblyInfo assemblyDelegate;
		public ModuleInfo moduleDelegate;
		public ClassInfo classDelegate;
		public FunctionInfo functionDelegate;
		public ProfileInfo()
		{
		}
		private void ProcessThread(XmlReader rdr)
		{
			bool continueProcessing = true;
			uint id = 0;
			uint handle = 0;
			uint win32id = 0;
			while (rdr.Read() && continueProcessing)
			{
				switch (rdr.NodeType)
				{
					case XmlNodeType.Element:
						if(rdr.Name == "id")
						{
							id = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "handle")
						{
							handle = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "win32id")
						{
							win32id = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						break;
					case XmlNodeType.EndElement:
						if(rdr.Name == "thread")
							continueProcessing = false;
						break;
				}
			}
			Thread(id, handle, win32id);
		}
		private void ProcessAppDomain(XmlReader rdr)
		{
			bool continueProcessing = true;
			uint id = 0;
			string name = "";
			uint processid = 0;
			while (rdr.Read() && continueProcessing)
			{
				switch (rdr.NodeType)
				{
					case XmlNodeType.Element:
						if(rdr.Name == "id")
						{
							id = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "name")
						{
							name = rdr.ReadString();
							break;
						}
						if(rdr.Name == "process")
						{
							processid = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						break;
					case XmlNodeType.EndElement:
						if(rdr.Name == "appdomain")
							continueProcessing = false;
						break;
				}
			}
			AppDomain(id, name, processid);
		}
		private void ProcessAssembly(XmlReader rdr)
		{
			bool continueProcessing = true;
			uint id = 0;
			string name = "";
			uint moduleid = 0;
			uint appdomainid = 0;
			while (rdr.Read() && continueProcessing)
			{
				switch (rdr.NodeType)
				{
					case XmlNodeType.Element:
						if(rdr.Name == "id")
						{
							id = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "name")
						{
							name = rdr.ReadString();
							break;
						}
						if(rdr.Name == "module")
						{
							moduleid = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "appdomain")
						{
							appdomainid = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						break;
					case XmlNodeType.EndElement:
						if(rdr.Name == "assembly")
							continueProcessing = false;
						break;
				}
			}
			Assembly(id, name, moduleid, appdomainid);
		}
		private void ProcessModule(XmlReader rdr)
		{
			bool continueProcessing = true;
			uint id = 0;
			string name = "";
			uint assemblyid = 0;
			uint address = 0;
			while (rdr.Read() && continueProcessing)
			{
				switch (rdr.NodeType)
				{
					case XmlNodeType.Element:
						if(rdr.Name == "id")
						{
							id = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "name")
						{
							name = rdr.ReadString();
							break;
						}
						if(rdr.Name == "assembly")
						{
							assemblyid = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "address")
						{
							address = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						break;
					case XmlNodeType.EndElement:
						if(rdr.Name == "module")
							continueProcessing = false;
						break;
				}
			}
			Module(id, name, assemblyid, address);
		}
		private void ProcessClass(XmlReader rdr)
		{
			bool continueProcessing = true;
			uint id = 0;
			string name = "";
			uint tokenid = 0;
			uint moduleid = 0;
			while (rdr.Read() && continueProcessing)
			{
				switch (rdr.NodeType)
				{
					case XmlNodeType.Element:
						if(rdr.Name == "id")
						{
							id = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "name")
						{
							name = rdr.ReadString();
							break;
						}
						if(rdr.Name == "token")
						{
							tokenid = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "module")
						{
							moduleid = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						break;
					case XmlNodeType.EndElement:
						if(rdr.Name == "class")
							continueProcessing = false;
						break;
				}
			}
			Class(id, name, tokenid, moduleid);
		}
		private void ProcessFunction(XmlReader rdr)
		{
			bool continueProcessing = true;
			uint id = 0;
			string name = "";
			uint tokenid = 0;
			uint classid = 0;
			uint moduleid = 0;
			uint size = 0;
			long address = 0;
			uint enter = 0;
			uint left = 0;
			uint tail = 0;
			uint unwind = 0;
			while (rdr.Read() && continueProcessing)
			{
				switch (rdr.NodeType)
				{
					case XmlNodeType.Element:
						if(rdr.Name == "id")
						{
							id = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "name")
						{
							name = rdr.ReadString();
							break;
						}
						if(rdr.Name == "token")
						{
							tokenid = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "class")
						{
							classid = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "module")
						{
							moduleid = Convert.ToUInt32(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "size")
						{
							size = Convert.ToUInt32(rdr.ReadString());
							break;
						}
						if(rdr.Name == "address")
						{
							address = Convert.ToInt64(rdr.ReadString(), 16);
							break;
						}
						if(rdr.Name == "enter")
						{
							enter = Convert.ToUInt32(rdr.ReadString());
							break;
						}
						if(rdr.Name == "left")
						{
							left = Convert.ToUInt32(rdr.ReadString());
							break;
						}
						if(rdr.Name == "tail")
						{
							tail = Convert.ToUInt32(rdr.ReadString());
							break;
						}
						if(rdr.Name == "unwind")
						{
							unwind = Convert.ToUInt32(rdr.ReadString());
							break;
						}
						break;
					case XmlNodeType.EndElement:
						if(rdr.Name == "function")
							continueProcessing = false;
						break;
				}
			}
			Function(id, name, tokenid, classid,
			         moduleid, size, address,
			         enter, left, tail, unwind);
		}
		public void ProcessData(XmlReader rdr)
		{
			//Read each node in the tree.
			while (rdr.Read())
			{
				switch (rdr.NodeType)
				{
					case XmlNodeType.Element:
						if(rdr.Name == "thread")
						{
							ProcessThread(rdr);
							break;
						}
						if(rdr.Name == "appdomain")
						{
							ProcessAppDomain(rdr);
							break;
						}
						if(rdr.Name == "assembly")
						{
							ProcessAssembly(rdr);
							break;
						}
						if(rdr.Name == "module")
						{
							ProcessModule(rdr);
							break;
						}
						if(rdr.Name == "class")
						{
							ProcessClass(rdr);
							break;
						}
						if(rdr.Name == "function")
						{
							ProcessFunction(rdr);
							break;
						}
						break;
					case XmlNodeType.Text:
						break;
					case XmlNodeType.CDATA:
						break;
					case XmlNodeType.ProcessingInstruction:
						break;
					case XmlNodeType.Comment:
						break;
					case XmlNodeType.Document:
						break;
					case XmlNodeType.Whitespace:
						break;
					case XmlNodeType.SignificantWhitespace:
						break;
					case XmlNodeType.EndElement:
						break;
				}
			}
		}
		public void Thread(uint id, uint handle, uint w32id)
		{
			if(threadDelegate != null)
				threadDelegate(id, handle, w32id);
		}
		public void AppDomain(uint id, string name, uint processid)
		{
			if(appdomainDelegate != null)
				appdomainDelegate(id, name, processid);
		}
		public void Assembly(uint id, string name, uint moduleid, uint appdomainid)
		{
			if(assemblyDelegate != null)
				assemblyDelegate(id, name, moduleid, appdomainid);
		}
		public void Module(uint id, string name, uint assemblyid, long baseaddress)
		{
			if(moduleDelegate != null)
				moduleDelegate(id, name, assemblyid, baseaddress);
		}
		public void Class(uint id, string name, uint token, uint moduleid)
		{
			if(classDelegate != null)
				classDelegate(id, name, token, moduleid);
		}
		public void Function(uint id, string name, uint token, uint classid,
			                 uint moduleid, uint size, long startaddress,
			                 uint enter, uint leave, uint tail, uint unwind)
		{
			if(functionDelegate != null)
				functionDelegate(id, name, token, classid,
					             moduleid, size, startaddress,
					             enter, leave, tail, unwind);
		}
	}
}
