//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Diagnostics;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Xml;

namespace WinGcp
{
	/// <summary>
	/// Summary description for GcpForm.
	/// </summary>
	public class GcpForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label fileToProfileLabel;
		private System.Windows.Forms.TextBox pathToFile;
		private System.Windows.Forms.Button browseButton;
		private System.Windows.Forms.OpenFileDialog openFileDialog;
		private System.Windows.Forms.TabControl profileInfoTabControl;
		private System.Windows.Forms.StatusBarPanel statusBarMessage;
		private System.Windows.Forms.StatusBarPanel statusBarDate;
		private System.Windows.Forms.StatusBarPanel statusBarTime;
		private System.Windows.Forms.TabPage threadPage;
		private System.Windows.Forms.TabPage appDomainPage;
		private System.Windows.Forms.TabPage assemblyPage;
		private System.Windows.Forms.TabPage modulePage;
		private System.Windows.Forms.TabPage classPage;
		private System.Windows.Forms.TabPage functionPage;
		private System.Windows.Forms.ListView threadListView;
		private System.Windows.Forms.ColumnHeader threadId;
		private System.Windows.Forms.ColumnHeader handle;
		private System.Windows.Forms.ColumnHeader win32Id;
		private System.Windows.Forms.Label threadCountLabel;
		private System.Windows.Forms.TextBox threadCount;
		private System.Windows.Forms.ListView appDomainListView;
		private System.Windows.Forms.ListView assemblyListView;
		private System.Windows.Forms.ListView moduleListView;
		private System.Windows.Forms.ListView classListView;
		private System.Windows.Forms.ListView functionListView;
		private System.Windows.Forms.TextBox appDomainCount;
		private System.Windows.Forms.Label appDomainCountLabel;
		private System.Windows.Forms.TextBox assemblyCount;
		private System.Windows.Forms.Label assemblyCountLabel;
		private System.Windows.Forms.TextBox moduleCount;
		private System.Windows.Forms.Label moduleCountLabel;
		private System.Windows.Forms.TextBox classCount;
		private System.Windows.Forms.Label classCountLabel;
		private System.Windows.Forms.TextBox functionCount;
		private System.Windows.Forms.Label functionCountLabel;
		private System.Windows.Forms.ColumnHeader appDomainId;
		private System.Windows.Forms.ColumnHeader appDomainName;
		private System.Windows.Forms.ColumnHeader appDomainProcessId;
		private System.Windows.Forms.ColumnHeader assemblyId;
		private System.Windows.Forms.ColumnHeader assemblyName;
		private System.Windows.Forms.ColumnHeader assemblyAppDomainId;
		private System.Windows.Forms.ColumnHeader assemblyModuleId;
		private System.Windows.Forms.ColumnHeader moduleId;
		private System.Windows.Forms.ColumnHeader moduleName;
		private System.Windows.Forms.ColumnHeader moduleAssemblyId;
		private System.Windows.Forms.ColumnHeader moduleBaseAddress;
		private System.Windows.Forms.ColumnHeader classId;
		private System.Windows.Forms.ColumnHeader className;
		private System.Windows.Forms.ColumnHeader classToken;
		private System.Windows.Forms.ColumnHeader classModuleId;
		private System.Windows.Forms.ColumnHeader functionId;
		private System.Windows.Forms.ColumnHeader FunctionName;
		private System.Windows.Forms.ColumnHeader functionToken;
		private System.Windows.Forms.ColumnHeader functionClassId;
		private System.Windows.Forms.ColumnHeader functionModuleId;
		private System.Windows.Forms.ColumnHeader functionSize;
		private System.Windows.Forms.ColumnHeader functionStartAddress;
		private System.Windows.Forms.ColumnHeader functionEnter;
		private System.Windows.Forms.ColumnHeader functionLeave;
		private System.Windows.Forms.ColumnHeader functionTail;
		private System.Windows.Forms.ColumnHeader functionUnwind;
		private System.Windows.Forms.Button startProfileButton;
		private System.Windows.Forms.ErrorProvider errorProvider;
		private System.Windows.Forms.StatusBar statusBar;
		private System.Windows.Forms.Timer timer;
		private System.Windows.Forms.TextBox arguments;
		private System.Windows.Forms.Label label1;
		private System.ComponentModel.IContainer components;
		private ProfileInfo profileInfoInterface;
		private Thread resultsThread;
		private TcpListener listeningClient;

		public GcpForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			errorProvider = new ErrorProvider();
			DateTime now = DateTime.Now;
			statusBarDate.Text = now.ToShortDateString();
			statusBarTime.Text = now.ToShortTimeString();
			timer.Enabled = true;

			profileInfoInterface = new ProfileInfo();
			profileInfoInterface.threadDelegate += new ProfileInfo.ThreadInfo(OnThreadInfo);
			profileInfoInterface.appdomainDelegate += new ProfileInfo.AppDomainInfo(OnAppDomainInfo);
			profileInfoInterface.assemblyDelegate += new ProfileInfo.AssemblyInfo(OnAssemblyInfo);
			profileInfoInterface.moduleDelegate += new ProfileInfo.ModuleInfo(OnModuleInfo);
			profileInfoInterface.classDelegate += new ProfileInfo.ClassInfo(OnClassInfo);
			profileInfoInterface.functionDelegate += new ProfileInfo.FunctionInfo(OnFunctionInfo);
			resultsThread = new Thread(new ThreadStart(ResultsThread));
			resultsThread.Start();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
				if(resultsThread != null)
				{
					if(listeningClient != null)
						listeningClient.Stop();
					resultsThread.Abort();
					resultsThread.Join();
					resultsThread = null;
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.fileToProfileLabel = new System.Windows.Forms.Label();
			this.pathToFile = new System.Windows.Forms.TextBox();
			this.browseButton = new System.Windows.Forms.Button();
			this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.profileInfoTabControl = new System.Windows.Forms.TabControl();
			this.threadPage = new System.Windows.Forms.TabPage();
			this.threadCount = new System.Windows.Forms.TextBox();
			this.threadCountLabel = new System.Windows.Forms.Label();
			this.threadListView = new System.Windows.Forms.ListView();
			this.threadId = new System.Windows.Forms.ColumnHeader();
			this.handle = new System.Windows.Forms.ColumnHeader();
			this.win32Id = new System.Windows.Forms.ColumnHeader();
			this.appDomainPage = new System.Windows.Forms.TabPage();
			this.appDomainCount = new System.Windows.Forms.TextBox();
			this.appDomainCountLabel = new System.Windows.Forms.Label();
			this.appDomainListView = new System.Windows.Forms.ListView();
			this.appDomainId = new System.Windows.Forms.ColumnHeader();
			this.appDomainName = new System.Windows.Forms.ColumnHeader();
			this.appDomainProcessId = new System.Windows.Forms.ColumnHeader();
			this.assemblyPage = new System.Windows.Forms.TabPage();
			this.assemblyCount = new System.Windows.Forms.TextBox();
			this.assemblyCountLabel = new System.Windows.Forms.Label();
			this.assemblyListView = new System.Windows.Forms.ListView();
			this.assemblyId = new System.Windows.Forms.ColumnHeader();
			this.assemblyName = new System.Windows.Forms.ColumnHeader();
			this.assemblyModuleId = new System.Windows.Forms.ColumnHeader();
			this.assemblyAppDomainId = new System.Windows.Forms.ColumnHeader();
			this.modulePage = new System.Windows.Forms.TabPage();
			this.moduleCount = new System.Windows.Forms.TextBox();
			this.moduleCountLabel = new System.Windows.Forms.Label();
			this.moduleListView = new System.Windows.Forms.ListView();
			this.moduleId = new System.Windows.Forms.ColumnHeader();
			this.moduleName = new System.Windows.Forms.ColumnHeader();
			this.moduleAssemblyId = new System.Windows.Forms.ColumnHeader();
			this.moduleBaseAddress = new System.Windows.Forms.ColumnHeader();
			this.classPage = new System.Windows.Forms.TabPage();
			this.classCount = new System.Windows.Forms.TextBox();
			this.classCountLabel = new System.Windows.Forms.Label();
			this.classListView = new System.Windows.Forms.ListView();
			this.classId = new System.Windows.Forms.ColumnHeader();
			this.className = new System.Windows.Forms.ColumnHeader();
			this.classToken = new System.Windows.Forms.ColumnHeader();
			this.classModuleId = new System.Windows.Forms.ColumnHeader();
			this.functionPage = new System.Windows.Forms.TabPage();
			this.functionCount = new System.Windows.Forms.TextBox();
			this.functionCountLabel = new System.Windows.Forms.Label();
			this.functionListView = new System.Windows.Forms.ListView();
			this.functionId = new System.Windows.Forms.ColumnHeader();
			this.FunctionName = new System.Windows.Forms.ColumnHeader();
			this.functionToken = new System.Windows.Forms.ColumnHeader();
			this.functionClassId = new System.Windows.Forms.ColumnHeader();
			this.functionModuleId = new System.Windows.Forms.ColumnHeader();
			this.functionSize = new System.Windows.Forms.ColumnHeader();
			this.functionStartAddress = new System.Windows.Forms.ColumnHeader();
			this.functionEnter = new System.Windows.Forms.ColumnHeader();
			this.functionLeave = new System.Windows.Forms.ColumnHeader();
			this.functionTail = new System.Windows.Forms.ColumnHeader();
			this.functionUnwind = new System.Windows.Forms.ColumnHeader();
			this.statusBar = new System.Windows.Forms.StatusBar();
			this.statusBarMessage = new System.Windows.Forms.StatusBarPanel();
			this.statusBarDate = new System.Windows.Forms.StatusBarPanel();
			this.statusBarTime = new System.Windows.Forms.StatusBarPanel();
			this.startProfileButton = new System.Windows.Forms.Button();
			this.errorProvider = new System.Windows.Forms.ErrorProvider();
			this.timer = new System.Windows.Forms.Timer(this.components);
			this.arguments = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.profileInfoTabControl.SuspendLayout();
			this.threadPage.SuspendLayout();
			this.appDomainPage.SuspendLayout();
			this.assemblyPage.SuspendLayout();
			this.modulePage.SuspendLayout();
			this.classPage.SuspendLayout();
			this.functionPage.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.statusBarMessage)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarDate)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarTime)).BeginInit();
			this.SuspendLayout();
			// 
			// fileToProfileLabel
			// 
			this.fileToProfileLabel.Location = new System.Drawing.Point(8, 16);
			this.fileToProfileLabel.Name = "fileToProfileLabel";
			this.fileToProfileLabel.Size = new System.Drawing.Size(72, 16);
			this.fileToProfileLabel.TabIndex = 0;
			this.fileToProfileLabel.Text = "Process";
			// 
			// pathToFile
			// 
			this.pathToFile.Location = new System.Drawing.Point(96, 8);
			this.pathToFile.Name = "pathToFile";
			this.pathToFile.Size = new System.Drawing.Size(328, 22);
			this.pathToFile.TabIndex = 1;
			this.pathToFile.Text = "";
			// 
			// browseButton
			// 
			this.browseButton.Location = new System.Drawing.Point(448, 8);
			this.browseButton.Name = "browseButton";
			this.browseButton.Size = new System.Drawing.Size(80, 23);
			this.browseButton.TabIndex = 2;
			this.browseButton.Text = "Browse . . .";
			this.browseButton.Click += new System.EventHandler(this.OnBrowse);
			// 
			// openFileDialog
			// 
			this.openFileDialog.Filter = "Executable Files (*.exe)|*.exe";
			this.openFileDialog.Title = "File To Profile";
			// 
			// profileInfoTabControl
			// 
			this.profileInfoTabControl.Controls.AddRange(new System.Windows.Forms.Control[] {
																								this.threadPage,
																								this.modulePage,
																								this.appDomainPage,
																								this.assemblyPage,
																								this.classPage,
																								this.functionPage});
			this.profileInfoTabControl.Location = new System.Drawing.Point(8, 112);
			this.profileInfoTabControl.Name = "profileInfoTabControl";
			this.profileInfoTabControl.SelectedIndex = 0;
			this.profileInfoTabControl.Size = new System.Drawing.Size(520, 312);
			this.profileInfoTabControl.TabIndex = 3;
			// 
			// threadPage
			// 
			this.threadPage.Controls.AddRange(new System.Windows.Forms.Control[] {
																					 this.threadCount,
																					 this.threadCountLabel,
																					 this.threadListView});
			this.threadPage.Location = new System.Drawing.Point(4, 25);
			this.threadPage.Name = "threadPage";
			this.threadPage.TabIndex = 0;
			this.threadPage.Text = "Thread";
			// 
			// threadCount
			// 
			this.threadCount.Location = new System.Drawing.Point(144, 8);
			this.threadCount.Name = "threadCount";
			this.threadCount.Size = new System.Drawing.Size(56, 22);
			this.threadCount.TabIndex = 2;
			this.threadCount.Text = "0";
			// 
			// threadCountLabel
			// 
			this.threadCountLabel.Location = new System.Drawing.Point(16, 16);
			this.threadCountLabel.Name = "threadCountLabel";
			this.threadCountLabel.Size = new System.Drawing.Size(96, 16);
			this.threadCountLabel.TabIndex = 1;
			this.threadCountLabel.Text = "Thread Count";
			// 
			// threadListView
			// 
			this.threadListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																							 this.threadId,
																							 this.handle,
																							 this.win32Id});
			this.threadListView.Location = new System.Drawing.Point(8, 40);
			this.threadListView.Name = "threadListView";
			this.threadListView.Size = new System.Drawing.Size(496, 240);
			this.threadListView.TabIndex = 0;
			this.threadListView.View = System.Windows.Forms.View.Details;
			// 
			// threadId
			// 
			this.threadId.Text = "ID";
			this.threadId.Width = 80;
			// 
			// handle
			// 
			this.handle.Text = "Handle";
			this.handle.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
			this.handle.Width = 100;
			// 
			// win32Id
			// 
			this.win32Id.Text = "Win32 ID";
			this.win32Id.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
			this.win32Id.Width = 100;
			// 
			// appDomainPage
			// 
			this.appDomainPage.Controls.AddRange(new System.Windows.Forms.Control[] {
																						this.appDomainCount,
																						this.appDomainCountLabel,
																						this.appDomainListView});
			this.appDomainPage.Location = new System.Drawing.Point(4, 25);
			this.appDomainPage.Name = "appDomainPage";
			this.appDomainPage.TabIndex = 1;
			this.appDomainPage.Text = "AppDomain";
			// 
			// appDomainCount
			// 
			this.appDomainCount.Location = new System.Drawing.Point(144, 8);
			this.appDomainCount.Name = "appDomainCount";
			this.appDomainCount.Size = new System.Drawing.Size(56, 22);
			this.appDomainCount.TabIndex = 4;
			this.appDomainCount.Text = "0";
			// 
			// appDomainCountLabel
			// 
			this.appDomainCountLabel.Location = new System.Drawing.Point(16, 16);
			this.appDomainCountLabel.Name = "appDomainCountLabel";
			this.appDomainCountLabel.Size = new System.Drawing.Size(120, 16);
			this.appDomainCountLabel.TabIndex = 3;
			this.appDomainCountLabel.Text = "AppDomain Count";
			// 
			// appDomainListView
			// 
			this.appDomainListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																								this.appDomainId,
																								this.appDomainName,
																								this.appDomainProcessId});
			this.appDomainListView.Location = new System.Drawing.Point(8, 40);
			this.appDomainListView.Name = "appDomainListView";
			this.appDomainListView.Size = new System.Drawing.Size(496, 240);
			this.appDomainListView.TabIndex = 0;
			this.appDomainListView.View = System.Windows.Forms.View.Details;
			// 
			// appDomainId
			// 
			this.appDomainId.Text = "Id";
			this.appDomainId.Width = 80;
			// 
			// appDomainName
			// 
			this.appDomainName.Text = "Name";
			this.appDomainName.Width = 250;
			// 
			// appDomainProcessId
			// 
			this.appDomainProcessId.Text = "Process Id";
			this.appDomainProcessId.Width = 100;
			// 
			// assemblyPage
			// 
			this.assemblyPage.Controls.AddRange(new System.Windows.Forms.Control[] {
																					   this.assemblyCount,
																					   this.assemblyCountLabel,
																					   this.assemblyListView});
			this.assemblyPage.Location = new System.Drawing.Point(4, 25);
			this.assemblyPage.Name = "assemblyPage";
			this.assemblyPage.TabIndex = 2;
			this.assemblyPage.Text = "Assembly";
			// 
			// assemblyCount
			// 
			this.assemblyCount.Location = new System.Drawing.Point(144, 8);
			this.assemblyCount.Name = "assemblyCount";
			this.assemblyCount.Size = new System.Drawing.Size(56, 22);
			this.assemblyCount.TabIndex = 4;
			this.assemblyCount.Text = "0";
			// 
			// assemblyCountLabel
			// 
			this.assemblyCountLabel.Location = new System.Drawing.Point(16, 16);
			this.assemblyCountLabel.Name = "assemblyCountLabel";
			this.assemblyCountLabel.Size = new System.Drawing.Size(112, 16);
			this.assemblyCountLabel.TabIndex = 3;
			this.assemblyCountLabel.Text = "Assembly Count";
			// 
			// assemblyListView
			// 
			this.assemblyListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																							   this.assemblyId,
																							   this.assemblyName,
																							   this.assemblyModuleId,
																							   this.assemblyAppDomainId});
			this.assemblyListView.Location = new System.Drawing.Point(8, 40);
			this.assemblyListView.Name = "assemblyListView";
			this.assemblyListView.Size = new System.Drawing.Size(496, 240);
			this.assemblyListView.TabIndex = 1;
			this.assemblyListView.View = System.Windows.Forms.View.Details;
			// 
			// assemblyId
			// 
			this.assemblyId.Text = "Id";
			this.assemblyId.Width = 80;
			// 
			// assemblyName
			// 
			this.assemblyName.Text = "Name";
			this.assemblyName.Width = 200;
			// 
			// assemblyModuleId
			// 
			this.assemblyModuleId.Text = "Module";
			this.assemblyModuleId.Width = 100;
			// 
			// assemblyAppDomainId
			// 
			this.assemblyAppDomainId.Text = "AppDomain";
			this.assemblyAppDomainId.Width = 100;
			// 
			// modulePage
			// 
			this.modulePage.Controls.AddRange(new System.Windows.Forms.Control[] {
																					 this.moduleCount,
																					 this.moduleCountLabel,
																					 this.moduleListView});
			this.modulePage.Location = new System.Drawing.Point(4, 25);
			this.modulePage.Name = "modulePage";
			this.modulePage.TabIndex = 3;
			this.modulePage.Text = "Module";
			// 
			// moduleCount
			// 
			this.moduleCount.Location = new System.Drawing.Point(144, 8);
			this.moduleCount.Name = "moduleCount";
			this.moduleCount.Size = new System.Drawing.Size(56, 22);
			this.moduleCount.TabIndex = 4;
			this.moduleCount.Text = "0";
			// 
			// moduleCountLabel
			// 
			this.moduleCountLabel.Location = new System.Drawing.Point(16, 16);
			this.moduleCountLabel.Name = "moduleCountLabel";
			this.moduleCountLabel.Size = new System.Drawing.Size(96, 16);
			this.moduleCountLabel.TabIndex = 3;
			this.moduleCountLabel.Text = "Module Count";
			// 
			// moduleListView
			// 
			this.moduleListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																							 this.moduleId,
																							 this.moduleName,
																							 this.moduleAssemblyId,
																							 this.moduleBaseAddress});
			this.moduleListView.Location = new System.Drawing.Point(8, 32);
			this.moduleListView.Name = "moduleListView";
			this.moduleListView.Size = new System.Drawing.Size(496, 240);
			this.moduleListView.TabIndex = 1;
			this.moduleListView.View = System.Windows.Forms.View.Details;
			// 
			// moduleId
			// 
			this.moduleId.Text = "Id";
			this.moduleId.Width = 80;
			// 
			// moduleName
			// 
			this.moduleName.Text = "Name";
			this.moduleName.Width = 230;
			// 
			// moduleAssemblyId
			// 
			this.moduleAssemblyId.Text = "Assembly";
			this.moduleAssemblyId.Width = 80;
			// 
			// moduleBaseAddress
			// 
			this.moduleBaseAddress.Text = "Base Address";
			this.moduleBaseAddress.Width = 80;
			// 
			// classPage
			// 
			this.classPage.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.classCount,
																					this.classCountLabel,
																					this.classListView});
			this.classPage.Location = new System.Drawing.Point(4, 25);
			this.classPage.Name = "classPage";
			this.classPage.TabIndex = 4;
			this.classPage.Text = "Class";
			// 
			// classCount
			// 
			this.classCount.Location = new System.Drawing.Point(144, 8);
			this.classCount.Name = "classCount";
			this.classCount.Size = new System.Drawing.Size(56, 22);
			this.classCount.TabIndex = 4;
			this.classCount.Text = "0";
			// 
			// classCountLabel
			// 
			this.classCountLabel.Location = new System.Drawing.Point(16, 16);
			this.classCountLabel.Name = "classCountLabel";
			this.classCountLabel.Size = new System.Drawing.Size(96, 16);
			this.classCountLabel.TabIndex = 3;
			this.classCountLabel.Text = "Class Count";
			// 
			// classListView
			// 
			this.classListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																							this.classId,
																							this.className,
																							this.classToken,
																							this.classModuleId});
			this.classListView.Location = new System.Drawing.Point(8, 40);
			this.classListView.Name = "classListView";
			this.classListView.Size = new System.Drawing.Size(496, 240);
			this.classListView.TabIndex = 1;
			this.classListView.View = System.Windows.Forms.View.Details;
			// 
			// classId
			// 
			this.classId.Text = "Id";
			this.classId.Width = 80;
			// 
			// className
			// 
			this.className.Text = "Name";
			this.className.Width = 250;
			// 
			// classToken
			// 
			this.classToken.Text = "Token";
			// 
			// classModuleId
			// 
			this.classModuleId.Text = "Module";
			// 
			// functionPage
			// 
			this.functionPage.Controls.AddRange(new System.Windows.Forms.Control[] {
																					   this.functionCount,
																					   this.functionCountLabel,
																					   this.functionListView});
			this.functionPage.Location = new System.Drawing.Point(4, 25);
			this.functionPage.Name = "functionPage";
			this.functionPage.TabIndex = 5;
			this.functionPage.Text = "Function";
			// 
			// functionCount
			// 
			this.functionCount.Location = new System.Drawing.Point(144, 8);
			this.functionCount.Name = "functionCount";
			this.functionCount.Size = new System.Drawing.Size(56, 22);
			this.functionCount.TabIndex = 4;
			this.functionCount.Text = "0";
			// 
			// functionCountLabel
			// 
			this.functionCountLabel.Location = new System.Drawing.Point(16, 16);
			this.functionCountLabel.Name = "functionCountLabel";
			this.functionCountLabel.Size = new System.Drawing.Size(96, 16);
			this.functionCountLabel.TabIndex = 3;
			this.functionCountLabel.Text = "Function Count";
			// 
			// functionListView
			// 
			this.functionListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																							   this.functionId,
																							   this.FunctionName,
																							   this.functionToken,
																							   this.functionClassId,
																							   this.functionModuleId,
																							   this.functionSize,
																							   this.functionStartAddress,
																							   this.functionEnter,
																							   this.functionLeave,
																							   this.functionTail,
																							   this.functionUnwind});
			this.functionListView.Location = new System.Drawing.Point(8, 40);
			this.functionListView.Name = "functionListView";
			this.functionListView.Size = new System.Drawing.Size(496, 240);
			this.functionListView.TabIndex = 1;
			this.functionListView.View = System.Windows.Forms.View.Details;
			// 
			// functionId
			// 
			this.functionId.Text = "Id";
			// 
			// FunctionName
			// 
			this.FunctionName.Text = "Name";
			this.FunctionName.Width = 200;
			// 
			// functionToken
			// 
			this.functionToken.Text = "Token";
			// 
			// functionClassId
			// 
			this.functionClassId.Text = "Class";
			// 
			// functionModuleId
			// 
			this.functionModuleId.Text = "Module";
			// 
			// functionSize
			// 
			this.functionSize.Text = "Size";
			// 
			// functionStartAddress
			// 
			this.functionStartAddress.Text = "Start";
			// 
			// functionEnter
			// 
			this.functionEnter.Text = "Enter";
			this.functionEnter.Width = 30;
			// 
			// functionLeave
			// 
			this.functionLeave.Text = "Leave";
			this.functionLeave.Width = 30;
			// 
			// functionTail
			// 
			this.functionTail.Text = "Tail";
			this.functionTail.Width = 30;
			// 
			// functionUnwind
			// 
			this.functionUnwind.Text = "Unwind";
			this.functionUnwind.Width = 30;
			// 
			// statusBar
			// 
			this.statusBar.Location = new System.Drawing.Point(0, 434);
			this.statusBar.Name = "statusBar";
			this.statusBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						 this.statusBarMessage,
																						 this.statusBarDate,
																						 this.statusBarTime});
			this.statusBar.ShowPanels = true;
			this.statusBar.Size = new System.Drawing.Size(536, 22);
			this.statusBar.TabIndex = 4;
			// 
			// statusBarMessage
			// 
			this.statusBarMessage.Width = 385;
			// 
			// statusBarDate
			// 
			this.statusBarDate.MinWidth = 65;
			this.statusBarDate.Width = 65;
			// 
			// statusBarTime
			// 
			this.statusBarTime.MinWidth = 65;
			this.statusBarTime.Width = 65;
			// 
			// startProfileButton
			// 
			this.startProfileButton.Location = new System.Drawing.Point(8, 72);
			this.startProfileButton.Name = "startProfileButton";
			this.startProfileButton.TabIndex = 5;
			this.startProfileButton.Text = "Start";
			this.startProfileButton.Click += new System.EventHandler(this.OnStart);
			// 
			// timer
			// 
			this.timer.Interval = 30000;
			this.timer.Tick += new System.EventHandler(this.OnTimer);
			// 
			// arguments
			// 
			this.arguments.Location = new System.Drawing.Point(96, 40);
			this.arguments.Name = "arguments";
			this.arguments.Size = new System.Drawing.Size(328, 22);
			this.arguments.TabIndex = 7;
			this.arguments.Text = "";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 48);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(72, 16);
			this.label1.TabIndex = 6;
			this.label1.Text = "Arguments";
			// 
			// GcpForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(6, 15);
			this.ClientSize = new System.Drawing.Size(536, 456);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.arguments,
																		  this.label1,
																		  this.startProfileButton,
																		  this.statusBar,
																		  this.profileInfoTabControl,
																		  this.browseButton,
																		  this.pathToFile,
																		  this.fileToProfileLabel});
			this.Name = "GcpForm";
			this.Text = "Windows General  Code Profiler";
			this.profileInfoTabControl.ResumeLayout(false);
			this.threadPage.ResumeLayout(false);
			this.appDomainPage.ResumeLayout(false);
			this.assemblyPage.ResumeLayout(false);
			this.modulePage.ResumeLayout(false);
			this.classPage.ResumeLayout(false);
			this.functionPage.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.statusBarMessage)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarDate)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarTime)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new GcpForm());
		}

		private void OnBrowse(object sender, System.EventArgs e)
		{
			DialogResult result = openFileDialog.ShowDialog();
			if(result == DialogResult.OK)
			{
				pathToFile.Text = openFileDialog.FileName;
			}
		}

		private void OnStart(object sender, System.EventArgs e)
		{
			if(!File.Exists(pathToFile.Text))
			{
				errorProvider.SetError(pathToFile, "Please enter an valid path.");
				return;
			}
			else
			{
				threadCount.Text = 
				appDomainCount.Text = 
				assemblyCount.Text = 
				moduleCount.Text = 
				classCount.Text = 
				functionCount.Text = "0";

				threadListView.Items.Clear();
				appDomainListView.Items.Clear();
				assemblyListView.Items.Clear();
				moduleListView.Items.Clear();
				classListView.Items.Clear();
				functionListView.Items.Clear();

				errorProvider.SetError(pathToFile, "");
				ProcessStartInfo psi;
				if(arguments.Text.Length > 0)
					psi = new ProcessStartInfo(pathToFile.Text, arguments.Text);
				else
					psi = new ProcessStartInfo(pathToFile.Text);
				psi.EnvironmentVariables.Add("DBG_PRF_LOG", "1");
				psi.EnvironmentVariables.Add("Cor_Enable_Profiling", "1");
				psi.EnvironmentVariables.Add("COR_PROFILER", "{01568439-E2BA-4434-8ACC-816239E8B8B5}");
				// psi.EnvironmentVariables.Add("COR_PROFILER_SERVER_PORT", "8085");
				// psi.EnvironmentVariables.Add("COR_PROFILER_SERVER", "localhost");
				psi.UseShellExecute = false;
				int pos = pathToFile.Text.LastIndexOf("\\");
				psi.WorkingDirectory = pathToFile.Text.Substring(0, pos);
				Process p = Process.Start(psi);
			}
		}

		private void OnTimer(object sender, System.EventArgs e)
		{
			DateTime now = DateTime.Now;
			statusBarDate.Text = now.ToShortDateString();
			statusBarTime.Text = now.ToShortTimeString();
		}
		private void ResultsThread()
		{
			listeningClient = new TcpListener(8085);
			listeningClient.Start();
			try
			{
				bool continueProcessing = true;
				while(continueProcessing)
				{
					Socket s = listeningClient.AcceptSocket();
					NetworkStream ns = new NetworkStream(s);
					XmlTextReader channel = new XmlTextReader(ns);
					profileInfoInterface.ProcessData(channel);
				}
			}
			catch(Exception e)
			{
				// SocketException when listeningClient.Stop is called
				// during an AcceptSocket.
				// "A blocking operation was interrpted by a call to
				// WSACancelBlockingCall"
				Debug.WriteLine(e);
			}
			finally
			{
				listeningClient.Stop();
			}
		}
		private void OnThreadInfo(uint id, uint handle, uint w32id)
		{
			ListViewItem item = threadListView.Items.Add(string.Format("0x{0:X}", id));
			item.SubItems.Add(string.Format("0x{0:X}", handle));
			item.SubItems.Add(string.Format("0x{0:X}", w32id));
			threadCount.Text = (Convert.ToInt32(threadCount.Text) + 1).ToString();
		}
		private void OnAppDomainInfo(uint id, string name, uint processid)
		{
			ListViewItem item = appDomainListView.Items.Add(string.Format("0x{0:X}", id));
			item.SubItems.Add(name);
			item.SubItems.Add(string.Format("0x{0:X}", processid));
			appDomainCount.Text = (Convert.ToInt32(appDomainCount.Text) + 1).ToString();
		}
		private void OnAssemblyInfo(uint id, string name, uint moduleid, uint appdomainid)
		{
			ListViewItem item = assemblyListView.Items.Add(string.Format("0x{0:X}", id));
			item.SubItems.Add(name);
			item.SubItems.Add(string.Format("0x{0:X}", moduleid));
			item.SubItems.Add(string.Format("0x{0:X}", appdomainid));
			assemblyCount.Text = (Convert.ToInt32(assemblyCount.Text) + 1).ToString();
		}
		private void OnModuleInfo(uint id, string name, uint assemblyid, long baseaddress)
		{
			ListViewItem item = moduleListView.Items.Add(string.Format("0x{0:X}", id));
			item.SubItems.Add(name);
			item.SubItems.Add(string.Format("0x{0:X}", assemblyid));
			item.SubItems.Add(string.Format("0x{0:X}", baseaddress));
			moduleCount.Text = (Convert.ToInt32(moduleCount.Text) + 1).ToString();
		}
		private void OnClassInfo(uint id, string name, uint token, uint moduleid)
		{
			ListViewItem item = classListView.Items.Add(string.Format("0x{0:X}", id));
			item.SubItems.Add(name);
			item.SubItems.Add(string.Format("0x{0:X}", token));
			item.SubItems.Add(string.Format("0x{0:X}", moduleid));
			classCount.Text = (Convert.ToInt32(classCount.Text) + 1).ToString();
		}
		private void OnFunctionInfo(uint id, string name, uint token, uint classid,
			                        uint moduleid, uint size, long startaddress,
			                        uint enter, uint leave, uint tail, uint unwind)
		{
			ListViewItem item = functionListView.Items.Add(string.Format("0x{0:X}", id));
			item.SubItems.Add(name);
			item.SubItems.Add(string.Format("0x{0:X}", token));
			item.SubItems.Add(string.Format("0x{0:X}", classid));
			item.SubItems.Add(string.Format("0x{0:X}", moduleid));
			item.SubItems.Add(size.ToString());
			item.SubItems.Add(string.Format("0x{0:X}", startaddress));
			item.SubItems.Add(enter.ToString());
			item.SubItems.Add(leave.ToString());
			item.SubItems.Add(tail.ToString());
			item.SubItems.Add(unwind.ToString());
			functionCount.Text = (Convert.ToInt32(functionCount.Text) + 1).ToString();
		}
	}
}
