//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
/****************************************************************************************
 * Copyright (c) 1999 Microsoft Corporation.  All Rights Reserved.
 *
 * File:
 *  ProfilerHelper.h
 *
 * Description:
 *	
 *
 *
 ***************************************************************************************/
#ifndef __PROFILERHELPER_H__
#define __PROFILERHELPER_H__


class ObjectLock 
{
	public:
	
    	ObjectLock( CRITICAL_SECTION &cs );
		~ObjectLock();
	private:
    	CRITICAL_SECTION &lock;
};

/***************************************************************************************
 ********************                                               ********************
 ********************              Type Definitions                 ********************
 ********************                                               ********************
 ***************************************************************************************/

enum TransitionState
{ 
	UNINITIALIZED,
	MANAGED, 
	UNMANAGED
};

enum StackAction
{ 
	PUSH, 
	POP 
};

enum FunctionCounters
{ 
	ENTER, 
	LEAVE, 
	TAIL, 
	UNWIND
};

enum JitStatus
{ 
	NONE,
	JITTED, 
	PREJITTED,
};

/***************************************************************************************
 ********************                                               ********************
 ********************             StackBaseInfo Declaration         ********************
 ********************                                               ********************
 ***************************************************************************************/
class StackBaseInfo
{
	public:
    
    	StackBaseInfo( ULONG id );         
        virtual ~StackBaseInfo();
                
        
	public:
            
         StackBaseInfo *Clone();
		 void Dump(std::wostringstream & out) const;
        
        
 	public:
    
    	ULONG m_id;

    
}; // StackBaseInfo


/***************************************************************************************
 ********************                                               ********************
 ********************             BaseInfo Declaration              ********************
 ********************                                               ********************
 ***************************************************************************************/
class BaseInfo
{
	public:
		BaseInfo();
    	BaseInfo( ULONG id );         
        virtual ~BaseInfo();
                
        
	public:
            
		virtual void Dump(std::wostringstream & out) const;
        BOOL Compare( ULONG key );
        
        
 	public:
    
    	ULONG m_id;
        BOOL  m_isValid;
		std::wstring m_name;
    
}; // BaseInfo

 
/***************************************************************************************
 ********************                                               ********************
 ********************            AppDomainInfo Declaration          ********************
 ********************                                               ********************
 ***************************************************************************************/
class AppDomainInfo :
	public BaseInfo
{
	public:

		AppDomainInfo();
		AppDomainInfo( AppDomainID appDomainID );
   		virtual ~AppDomainInfo();
      

	public:
    
    	void Dump(std::wostringstream & out) const;        
        
        
	public:	
  	  	       	
		ProcessID m_processID;			
                       
}; // AppDomainInfo



/***************************************************************************************
 ********************                                               ********************
 ********************            AssemblyInfo  Declaration          ********************
 ********************                                               ********************
 ***************************************************************************************/
class AssemblyInfo :
	public BaseInfo
{
	public:

		AssemblyInfo( );
		AssemblyInfo( AssemblyID assemblyID );
   		virtual ~AssemblyInfo();
      

	public:
    
    	void Dump(std::wostringstream & out) const;        
        
        
	public:	
  	  	       	
		ModuleID	m_moduleID;
		AppDomainID	m_appDomainID;
                       
}; // AssemblyInfo


/***************************************************************************************
 ********************                                               ********************
 ********************            ModuleInfo Declaration             ********************
 ********************                                               ********************
 ***************************************************************************************/
class ModuleInfo :
	public BaseInfo
{
	public:

		ModuleInfo( );
		ModuleInfo( ModuleID moduleID );
   		virtual ~ModuleInfo();
      

	public:
    
    	void Dump(std::wostringstream & out) const;        
        
        
	public:	
  	  	       	
		AssemblyID		m_assemblyID;	
        IMetaDataImport	*m_pMDImport;	
		LPCBYTE			m_pBaseLoadAddress;        
                       
}; // ModuleInfo


/***************************************************************************************
 ********************                                               ********************
 ********************             ClassInfo Declaration             ********************
 ********************                                               ********************
 ***************************************************************************************/
class ClassInfo :
	public BaseInfo
{
	public:

		ClassInfo( );
		ClassInfo( ClassID classID );
   		virtual ~ClassInfo();
      

	public:
    
    	void Dump(std::wostringstream & out) const;        
        
        
	public:	
  	  	       
		mdTypeDef m_token;
       	ModuleID  m_moduleID;	
                       
}; // ClassInfo


/***************************************************************************************
 ********************                                               ********************
 ********************            ThreadInfo Declaration             ********************
 ********************                                               ********************
 ***************************************************************************************/
class ThreadInfo :
	public BaseInfo
{
	public:

		ThreadInfo();
		ThreadInfo( ThreadID threadID );         
		virtual ~ThreadInfo();
        

	public:
    
    	void Dump(std::wostringstream & out) const;
                
        
	public:	
  	  		
        // 
		// Once a thread dies, it is possible for its ID to be re-used
		// We will remeber the originalID in order to print the information
		// for each thread in the end of the run.
		//
		DWORD m_originalTID;
        
		HANDLE					m_hThread;
		TransitionState			m_status;
        DWORD 					m_win32ThreadID;
		std::stack<StackBaseInfo> m_latestUnwoundFunction;
}; // ThreadInfo


/***************************************************************************************
 ********************                                               ********************
 ********************          FunctionInfo Declaration             ********************
 ********************                                               ********************
 ***************************************************************************************/
class FunctionInfo :
	public BaseInfo
{
	public:

		FunctionInfo( );
		FunctionInfo( FunctionID functionID );
   		virtual ~FunctionInfo();
      

	public:
    
    	void Dump(std::wostringstream & out) const;     
        
        
	public:	
  	  	                        	
		// function counters
		LONG	 	m_enter;
		LONG	 	m_left;
		LONG	 	m_tail;
		LONG	 	m_unwind;

		// jit related information
		JitStatus	m_jitStatus;
		BOOL	 	m_isPitched;

        // other info
        mdToken  	m_token;               
        ClassID  	m_classID;
        ModuleID 	m_moduleID; 
        ULONG    	m_codeSize;
		LPCBYTE  	m_pStartAddress;
        
}; // FunctionInfo


/***************************************************************************************
 ********************                                               ********************
 ********************              PrfInfo Declaration              ********************
 ********************                                               ********************
 ***************************************************************************************/
class PrfInfo
{	         
    public:
    
        PrfInfo();                     
        virtual ~PrfInfo();                      
                      
                   
	// methods
   	
   	public:
                    
		// tables
		void DumpTables();
		void Failure( const char *message = NULL );

        // class methods
        void AddClass( ClassID classID );
        void RemoveClass( ClassID classID );

    	// thread methods
    	void AddThread( ThreadID threadID );                        
    	void RemoveThread( ThreadID threadID );
    	void UpdateTransitionState( TransitionState state );                        
		void UpdateOSThreadID( ThreadID managedThreadID, DWORD osThreadID );
		void UpdateUnwindStack( FunctionID *functionID, StackAction action );
        
        // module methods
        void AddModule( ModuleID moduleID );         		
        void RemoveModule( ModuleID moduleID );         		
		void ReleaseModuleMDInterface( ModuleID moduleID );
		void UpdateAssemblyInModule( ModuleID moduleID, AssemblyID assemblyID );

        // function methods
        void AddFunction( FunctionID functionID ); 
        void UpdateFunction( FunctionID functionID, JitStatus status ); 
        void RemoveFunction( FunctionID functionID );
		void MarkFunctionAsPitched( FunctionID functionID );
        void UpdateEnterExitCounters( FunctionID functionID, FunctionCounters which );
 
        // assembly methods
        void AddAssembly( AssemblyID assemblyID );                                    
        void RemoveAssembly( AssemblyID assemblyID );                                    

        // appdomain methods
        void AddAppDomain( AppDomainID appDomainID );
        void RemoveAppDomain( AppDomainID appDomainID );


  	private:
		PrfInfo(const PrfInfo& p);
		PrfInfo& operator=(const PrfInfo& p);
        static void _GetThreadInfoHelper( ICorProfilerInfo *pi, ThreadInfo *ppThreadInfo );
        static void _GetAssemblyInfoHelper( ICorProfilerInfo *pi, AssemblyInfo *pAssemblyInfo );        
        static void _GetModuleInfoHelper( ICorProfilerInfo *pi, ModuleInfo *pModuleInfo );           
        static void _GetAppDomainInfoHelper( ICorProfilerInfo *pi, AppDomainInfo *pAppDomainInfo );
        void _GetFunctionInfoHelper( ICorProfilerInfo *pi, FunctionInfo *pFunctionInfo );
    	void _GetClassInfoHelper( ICorProfilerInfo *pi, ClassInfo *pClassInfo );
		void _SetAppDomainName( ICorProfilerInfo *pi, AppDomainID AppDomainID );


	// data memebers

    protected:    

        DWORD m_dwEventMask;
        ICorProfilerInfo *m_pProfilerInfo;

        // tables
		std::map<ThreadID, ThreadInfo> m_threadTable;
		typedef std::map<ThreadID, ThreadInfo>::const_iterator ConstantThreadIterator;
		typedef std::map<ThreadID, ThreadInfo>::iterator ThreadIterator;

		std::map<AppDomainID, AppDomainInfo> m_appDomainTable;
		typedef std::map<AppDomainID, AppDomainInfo>::const_iterator ConstantAppDomainIterator;
		typedef std::map<AppDomainID, AppDomainInfo>::iterator AppDomainIterator;

		std::map<AssemblyID, AssemblyInfo> m_assemblyTable;
		typedef std::map<AssemblyID, AssemblyInfo>::const_iterator ConstantAssemblyIterator;
		typedef std::map<AssemblyID, AssemblyInfo>::iterator AssemblyIterator;

		std::map<ModuleID, ModuleInfo> m_moduleTable;
		typedef std::map<ModuleID, ModuleInfo>::const_iterator ConstantModuleIterator;
		typedef std::map<ModuleID, ModuleInfo>::iterator ModuleIterator;

		std::map<ClassID, ClassInfo> m_classTable;
		typedef std::map<ClassID, ClassInfo>::const_iterator ConstantClassIterator;
		typedef std::map<ClassID, ClassInfo>::iterator ClassIterator;

		std::map<FunctionID, FunctionInfo> m_functionTable;
		typedef std::map<FunctionID, FunctionInfo>::const_iterator ConstantFunctionIterator;
		typedef std::map<FunctionID, FunctionInfo>::iterator FunctionIterator;

	private:

		CRITICAL_SECTION threadCS;
		CRITICAL_SECTION classCS;
		CRITICAL_SECTION moduleCS;
		CRITICAL_SECTION functionCS;
		CRITICAL_SECTION assemblyCS;
		CRITICAL_SECTION appDomainCS;

}; // PrfInfo


#endif // __PROFILERHELPER_H___

// End of File